@@ -7,6 +7,8 @@ from typing import List, Generic, Mapping, Union, Callable, TypeVar, Tuple, Set,
 
 from makefun import create_function
 
+from pinject_design.di.proxiable import DelegatedVar
+
 T, U = TypeVar("T"), TypeVar("U")
 
 A = TypeVar("A")
@@ -92,6 +94,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
                 # logger.info(f"partial injection call :{pformat(call_kwargs)}")
                 full_kwargs = {**kwargs, **call_kwargs, **_kwargs}
                 return target_function(**full_kwargs)
+            # I guess we need to make a function for inner
 
             return inner
 
@@ -194,7 +197,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
         return Injected.mzip(*[kwargs[k] for k in keys]).map(lambda t: {k: v for k, v in zip(keys, t)})
 
     @property
-    def proxy(self):
+    def proxy(self)->DelegatedVar:
         """use this to modify injected variables freely without map.
         call eval() at the end to finish modification
         # it seems this thing is preventing from pickling?
@@ -458,7 +461,7 @@ def _injected_factory(**targets: Injected):
     return _impl
 
 
-def injected_factory(f):
+def injected_function(f):
     """
     any args starting with "_" is considered to be injected.
     :param f:
@@ -469,4 +472,8 @@ def injected_factory(f):
     for k in sig.parameters.keys():
         if k.startswith("_"):
             tgts[k] = Injected.by_name(k[1:])
-    return _injected_factory(**tgts)(f)
+    new_f = _injected_factory(**tgts)(f)
+    new_f.__name__ = f.__name__
+    return new_f
+    #return _injected_factory(**tgts)(f)
+
